# -*- coding: utf-8 -*-
from south.v2 import SchemaMigration

class Migration(SchemaMigration):

    def forwards(self, orm):
        pass

    def backwards(self, orm):
        pass

    models = {}

    complete_apps = ['common']
